#
# WiKicker
#
#  Configuration.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Configuration.pm,v 1.4 2004/08/03 12:55:51 naney Exp $
#
#
package WiKicker::App::Configuration;

use 5.005;
use strict;
use WiKicker::Configuration;
use WiKicker::Properties::Default;

use vars qw(@ISA);

@ISA = qw(WiKicker::Configuration);

sub new {
  my ($class, %options) = @_;

  my $self = bless {}, $class;

  my $properties_file = '';
  if ($options{properties_file}) {
    $properties_file = $options{properties_file};
  }

  $self->_load_configuration($properties_file);

  return $self;
};

sub _load_configuration {
  my ($self, $properties_file) = @_;

  my $default_properties = WiKicker::Properties::Default->new;

  $self->{properties}
    = WiKicker::Properties->new(parent => $default_properties);
  if ($properties_file) {
    $self->{properties}->load_file($properties_file);
  }

  if ($properties_file) {
    my @path = split('/', $properties_file);
    pop(@path);
    $self->{wikicker_home} = join('/', @path);
  }
  else {
    $self->{wikicker_home} = '';
  }

}

sub wikicker_home {
  my $self = shift;
  return $self->{wikicker_home};
}

sub database_directory {
  my $self = shift;

  if ($self->properties->get('wikicker.database.directory')) {
    return $self->properties->get('wikicker.database.directory');
  }
  else {
    return $self->wikicker_home . '/var/database';
  }
}

1;
