#
# WiKicker
#
#  Cookie.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Cookie.pm,v 1.3 2004/01/15 09:12:45 naney Exp $
#
#

package WiKicker::CGI::Cookie;

use 5.005;
use strict;

use CGI::Cookie qw/-no_xhtml/;
use WiKicker::CGI::User;

# WiKicker:
#
#   id: UNIQUE_ID
#   un: user name
#   tz: timezone
#   ew: editor width
#   eh: editor height
#   
sub make_user_cookie {
  my ($user, $cookie_domain, $cookie_path) = @_;
  my $values = make_user_cookie_values($user) || {};
  my $cookie = new CGI::Cookie(-name => 'WiKicker',
                               -value => $values,
                               -expires => '+3M',
                               -domain => $cookie_domain,
                               -path => $cookie_path);
  return $cookie;
}

# WiKicker::User-> cookie
sub make_user_cookie_values {
  my $user = shift;

  my $result = {};
  if ($user->id) {
    $result->{id} = $user->id;
  }
  elsif ($ENV{UNIQUE_ID}) {
    $result->{id} = $ENV{UNIQUE_ID};
  }
  
  if ($user->name) {
    $result->{un} = $user->name;
  }
  if ($user->timezone) {
    $result->{tz} = $user->timezone;
  }
  $result->{ew} = $user->editor_width;
  $result->{eh} = $user->editor_height;
  $result->{pc} = $user->preferences_changed;

  return $result;
}

# cookie -> WiKicker::User
sub set_user {
  my ($cookies, $user) = @_;
  
  if (exists $cookies->{'WiKicker'}) {
    my %wiki_cookie = $cookies->{'WiKicker'}->value;


    if ($wiki_cookie{'id'}) {
      $user->id($wiki_cookie{'id'});
    }
    elsif ($ENV{UNIQUE_ID}) {
      $user->id($ENV{UNIQUE_ID});
    }
    
    if ($wiki_cookie{'un'}) {
      $user->name($wiki_cookie{'un'});
    }
    if ($wiki_cookie{'tz'}) {
      $user->timezone($wiki_cookie{'tz'});
    }
    if ($wiki_cookie{'ew'}) {
      $user->editor_width($wiki_cookie{'ew'});
    }
    if ($wiki_cookie{'eh'}) {
      $user->editor_height($wiki_cookie{'eh'});
    }
    if ($wiki_cookie{'pc'}) {
      $user->preferences_changed($wiki_cookie{'pc'});
    }
    else {
      $user->preferences_changed(0);
    }
  }
  
#   my $uploadimg = '';
#   if (exists $cookies{'uploadimg'}) {
#     my %uploadimg_cookie = $cookies{'uploadimg'}->value;
#     $uploadimg = $uploadimg_cookie{url};
#   }

}

1;
