#
# WiKicker
#
#  Export.pm
#
#
# Copyright (C) 2003,2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Export.pm,v 1.13 2005/04/04 14:06:13 naney Exp $
#
#

package WiKicker::CGI::Export;

use 5.005;
use strict;
use WiKicker::WikiPage::WikiPageHtmlFragmentVisitor;
use WiKicker::WikiPage::Env;
use WiKicker::HTML::LinkMaker;
use WiKicker::User;
use WiKicker::UTF8;

sub export_page {
  my ($page_name, $document_node, $request) = @_;

  my $file = $request->configuration->export_page($page_name);
  unless ($file) {
    return;
  }

  my $visitor = WiKicker::WikiPage::WikiPageHtmlFragmentVisitor->new;

  my $dummy_user = WiKicker::User->new;
  if ($request->configuration) {
    $dummy_user
      ->timezone($request->configuration->properties->get('user.timezone'));
  }
  else {
    $dummy_user->timezone('JST');
  }

  my $env = WiKicker::WikiPage::HtmlEnv
    ->new(user => $dummy_user,
          resolver => $request->environment->resolver,
          wikicker_url => $request->url,
          wri_factory => $request->environment->create_wri_factory,
          environment => $request->environment,
          query => $request->query,
          page_name => $request->name,
          localizer => $request,
          page_name_list => $request->environment->db->page_name_list,
          form_charset => 'EUC-JP');

  $env->link_maker(WiKicker::HTML::LinkMaker->new_from_html_env(env => $env));

  $visitor->env($env);
  $visitor->no_edit_section(1);

  $visitor->visit($document_node);

  unless (open(OUTPUT, ">$file")) {
    warn "Can't open export to $file";
    return;
  }
  my $content = $visitor->content;
  $content = WiKicker::UTF8::encode('euc', $content);
  print OUTPUT $content;
  close(OUTPUT);
}

1;
