#
# WiKicker
#
#  DataManager.pm
#
#
# Copyright (C) 2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: DataManager.pm,v 1.5 2005/03/06 09:07:19 naney Exp $
#
#

package WiKicker::Diki::DataManager;

use 5.005;
use strict;
use Carp;
use Log::Log4perl;
use WiKicker::Diki::PrimaryDatabase;
use WiKicker::Diki::SecondaryDatabase;

sub new {
  my ($class, %param) = @_;

  unless (exists $param{directory}) {
    croak "$class: No directory parameter";
  }

  unless (exists $param{environment}) {
    croak "$class: No environment parameter";
  }

  my $self = bless { directory => $param{directory} }, $class;
  if (exists $param{primary_directory} && $param{primary_directory} ne '') {
    $self->{primary_directory} = $param{primary_directory};
  }
  else {
    $self->{primary_directory} = $self->directory . '/primary';
  }
  $self->{environment} = $param{environment};

  $self->_initialize;

  return $self;
}

sub environment { shift->{environment} }

sub directory {
  shift->{directory};
}

sub primary_directory {
  return shift->{primary_directory};
}

sub secondary_directory {
  shift->directory . '/secondary';
}

sub primary {
  shift->{primary};
}

sub secondary {
  shift->{secondary};
}

sub _initialize {
  my $self = shift;

  $self->{primary} = WiKicker::Diki::PrimaryDatabase
    ->new(directory => $self->primary_directory);

  $self->{secondary} = WiKicker::Diki::SecondaryDatabase
    ->new(directory => $self->secondary_directory,
          environment => $self->environment);
}

sub get_recent_diaries {
  my ($self, $num_of_days, $max_update) = @_;

  my $diary_day_list = $self->secondary->get_recent_diaries($num_of_days);
  my $update_request
    = $self->primary->check_recent_diaries($diary_day_list, $num_of_days);
  if (defined $update_request) {
    $self->secondary->update($diary_day_list, $update_request);
    $diary_day_list->updated(1);
  }
  else {
    $self->secondary->check_diary_term($diary_day_list, $max_update);
  }

  return $diary_day_list;
}

sub get_article_diaries {
  my ($self, $range) = @_;

  my $diary_day_list = $self->secondary->get_article_diaries($range);
  my $update_request
    = $self->primary->check_article_diaries($diary_day_list, $range);
  if (defined $update_request) {
    $self->secondary->update($diary_day_list, $update_request);
  }
  else {
    $self->secondary->check_diary_term($diary_day_list, 1);
  }

  return $diary_day_list;
}

sub get_one_day_diaries {
  my ($self, $range, $max_update) = @_;

  my $diary_day_list = $self->secondary->get_one_day_diaries($range);
  my $update_request
    = $self->primary->check_one_day_diaries($diary_day_list, $range);
  if (defined $update_request) {
    $self->secondary->update($diary_day_list, $update_request);
  }
  else {
    $self->secondary->check_diary_term($diary_day_list, $max_update);
  }

  return $diary_day_list;
}

sub get_ten_day_diaries {
  my ($self, $range, $max_update) = @_;

  my $diary_day_list = $self->secondary->get_ten_day_diaries($range);
  my $update_request
    = $self->primary->check_ten_day_diaries($diary_day_list, $range);
  if (defined $update_request) {
    $self->secondary->update($diary_day_list, $update_request);
  }
  else {
    $self->secondary->check_diary_term($diary_day_list, $max_update);
  }

  return $diary_day_list;
}

sub get_keywords {
  my ($self, $term) = @_;

  my $keyword_list = $self->secondary->get_keywords($term);
  my $update_request
    = $self->primary->check_keywords($keyword_list);
  if (defined $update_request) {
    $self->secondary->update_keyword($keyword_list, $update_request);
  }
  else {
    $self->secondary->check_keyword_term($keyword_list);
  }
  return $keyword_list;
}

sub get_recent_keyword_diaries {
  my ($self, $name, $num_of_days) = @_;

  my $diary_day_list = $self->secondary->get_recent_keyword_diaries($name, $num_of_days);
  my $update_request
    = $self->primary->check_diaries($diary_day_list);
  if (defined $update_request) {
    $self->secondary->update($diary_day_list, $update_request);
  }

  return $diary_day_list;
}

sub check_keyword_files_update {
  my $self = shift;

  Log::Log4perl->get_logger(__PACKAGE__)->debug("check .dkk update");
  $self->secondary->check_keyword_files_update($self->primary);
}

sub term_list {
  my $self = shift;

  $self->secondary->term_list;
}

sub ten_day_list {
  my $self = shift;

  return $self->primary->ten_day_list;
}

1;
