#
# WiKicker
#
#  PrimaryYearDatabase.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: PrimaryMonthDatabase.pm,v 1.4 2005/03/06 09:06:16 naney Exp $
#
#

package WiKicker::Diki::PrimaryMonthDatabase;

use 5.005;
use strict;
use WiKicker::Time;
use Carp;

sub new {
  my ($class, %param) = @_;

  croak 'no directory' unless exists $param{directory};
  croak 'no month' unless exists $param{month};

  my $self = bless { directory => $param{directory},
                     month => $param{month} }, $class;
}

sub directory { shift->{directory} }

sub recent_diary_article_set {
  my ($self, $num_of_days) = @_;

  my $files = $self->_get_article_list_file_list;
  my @sorted = sort { $b cmp $a } @$files;

  my %date_map;
  my %result;
  for my $file_name (@sorted) {
    $file_name =~ /^(\d\d\d\d-\d\d-\d\d)/;
    my $date = $1;
    $date_map{$date}++;
    if (keys %date_map > $num_of_days) {
      return (scalar(keys %date_map), \%result);
    }
    my $article_id = substr($file_name, 0, length($file_name) - 4);
    my @stat = stat($self->directory . '/' . $file_name);
    $result{$article_id} = $stat[9];
  }
  return (scalar(keys %date_map), \%result);
}

sub article_diary_article_set {
  my ($self, $range) = @_;

  my $target_file_name = "$range.dkd";

  my %result = ();
  my $files = $self->_get_article_list_file_list;

  for my $file_name (@$files) {
    next if $file_name ne $target_file_name;

    my $article_id = substr($file_name, 0, length($file_name) - 4);
    my @stat = stat($self->directory . '/' . $file_name);
    $result{$article_id} = $stat[9];
  }
  return \%result;
}

sub one_day_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^\d\d\d\d-\d\d-(.+)$/;
  my $target_day = $1;

  my %result = ();
  my $files = $self->_get_article_list_file_list;

  for my $file_name (@$files) {
    $file_name =~ /^\d\d\d\d-\d\d-(\d\d)/;
    my $day = $1;
    next if $day ne $target_day;

    my $article_id = substr($file_name, 0, length($file_name) - 4);
    my @stat = stat($self->directory . '/' . $file_name);
    $result{$article_id} = $stat[9];
  }
  return \%result;
}

sub ten_day_diary_article_set {
  my ($self, $range) = @_;

  $range =~ /^\d\d\d\d-\d\d-(.+)$/;
  my $ten_day = $1;

  my %result = ();
  my $files = $self->_get_article_list_file_list;

  for my $file_name (@$files) {
    $file_name =~ /^\d\d\d\d-\d\d-(\d\d)/;
    my $day = $1;
    next unless WiKicker::Time::day_in_ten_day($day, $ten_day);

    my $article_id = substr($file_name, 0, length($file_name) - 4);
    my @stat = stat($self->directory . '/' . $file_name);
    $result{$article_id} = $stat[9];
  }
  return \%result;
}

sub ten_day_list {
  my $self = shift;
  my $files = $self->_get_article_list_file_list;

  my %result;
  for my $file_name (@{$self->_get_article_list_file_list}) {
    $file_name =~ /^(\d\d\d\d-\d\d)-(\d\d)/;
    my $ym = $1;
    my $ten_day = WiKicker::Time::day_to_ten_day($2);
    $result{"$ym-$ten_day"}++;
  }
  return [ keys %result ];
}

sub _get_article_list_file_list {
  my $self = shift;
  unless (-e $self->directory) {
    return [];
  }

  my @files;
  opendir(DIR, $self->directory) || croak "Can't open directory: $!";
  my $file;
  while (defined ($file = readdir(DIR))) {
    if ($file =~ /^\d\d\d\d-\d\d-\d\d-.*\.dkd$/) {
      push(@files, $file);
    }
  }
  close(DIR);
  return \@files;
}

sub get_article {
  my ($self, $article_id) = @_;

  my $file_name = $self->directory . "/$article_id.dkd";
  open(FILE, $file_name) || croak "Can't open file $file_name: $!";
  local $/;
  my $value = <FILE>;
  close(FILE);
  return $value;
}

1;
