#
# WiKicker
#
#  Environment.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Environment.pm,v 1.2 2004/08/03 12:55:52 naney Exp $
#
#
package WiKicker::DikiCGI::Environment;

use 5.005;
use strict;
use WiKicker::Environment;
use WiKicker::Diki::DataManager;
use WiKicker::WRI::Resolver;
use Carp;

use vars qw(@ISA);
@ISA = qw(WiKicker::Environment);

sub url {
  my $self = shift;
  if (exists $self->{_url}) {
    return $self->{_url};
  }

  unless (exists $ENV{SCRIPT_NAME}) {
    die 'No SCRIPT_NAME environment variable';
  }

  return $self->{_url} = $ENV{SCRIPT_NAME};
}

sub data_manager {
  my $self = shift;

  if ($self->{data_manager}) {
    return $self->{data_manager};
  }

  my %param = (directory => $self->configuration->database_directory,
               primary_directory => $self->configuration->primary_database_directory,
               environment => $self,
              );


  return $self->{data_manager} = WiKicker::Diki::DataManager->new(%param);
}

sub check_keyword_files_update {
  my $self = shift;

  $self->data_manager->check_keyword_files_update;
}

# tmp for HtmlEnv
sub db {
  my $self = shift;
  return $self->data_manager;
}

# remove relation circuit
sub destroy_data_manager {
  my $self = shift;

  $self->{data_manager} = undef;
}

# same as WiKicker::CGI::Request::maketext. @TODO refactoring.
sub maketext {
  my ($self, @param) = @_;

  unless (exists $self->{_language_handle}) {
    $self->{_language_handle} = WiKicker::L10N->get_handle;
  }

  if (5.006 <= $] && $] < 5.7) {
      return eval "no utf8; my \$tmp = \$self->{_language_handle}->maketext(\@param); \$tmp =~ /^(.*)\$/; \$1";
  }

  return $self->{_language_handle}->maketext(@param);
}

sub resolver {
  my $self = shift;

  if ($self->{resolver}) {
    return $self->{resolver};
  }
  else {
    $self->{resolver} = WiKicker::WRI::Resolver->new;
    $self->{resolver}->db(undef);
    $self->{resolver}->amazon_id($self->configuration->properties->get('Amazon.co.jp.AssociatesID'));
    return $self->{resolver};
  }
}


1;
