#
# WiKicker
#
#  DiaryPage.pm
#
#
# Copyright (C) 2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: DiaryPage.pm,v 1.8 2005/04/04 14:10:02 naney Exp $
#
#

package WiKicker::DikiCGI::Page::DiaryPage;

use 5.005;
use strict;
use WiKicker::DikiCGI::Page;

use vars qw(@ISA);
@ISA = qw(WiKicker::DikiCGI::Page);

sub do_read {
  my $self = shift;

  my $range_type = $self->request->command->range_type;
  return $self->_read_ten_day if $range_type eq 'ten-day';
  return $self->_read_one_day if $range_type eq 'day';
  return $self->_read_article if $range_type eq 'article';
#  return $self->_read_day_to_day if $range_type eq 'day-to-day';
  return $self->_read_recent;
}

sub _read_recent {
  my $self = shift;

  my $environment = $self->request->environment;
  my $diaries = $environment->data_manager->get_recent_diaries(5, $self->_max_update);
  if ($diaries->updated) {
    $environment->touch;
  }

  if ($self->request->command->format ne 'rss') {
    $self->request->attribute('diaries', $diaries);
    return 'RecentDiaries';
  }
  else {
    $self->_make_rss($diaries);
    return 'rss';
  }
}

sub _read_ten_day {
  my $self = shift;

  $self->request
    ->attribute('diaries',
                $self->_data_manager->
                get_ten_day_diaries($self->request->command->range,
                                    $self->_max_update));
  return 'TenDayDiaries';
}

# sub _read_day_to_day {
#   my $self = shift;

#   $self->request
#     ->attribute('diaries',
#                 $self->_data_manager->
#                 get_day_to_day_diaries($self->request->command->range,
#                                        $self->_max_update));

#   return 'DayToDayDiaries';
# }

sub _read_one_day {
  my $self = shift;

  $self->request
    ->attribute('diaries',
                $self->_data_manager
                ->get_one_day_diaries($self->request->command->range,
                                      $self->_max_update));

  return 'OneDayDiaries';
}

sub _read_article {
  my $self = shift;

  $self->request
    ->attribute('diaries',
                $self->_data_manager
                ->get_article_diaries($self->request->command->range));

  return 'ArticleDiaries';
}

sub _data_manager {
  my $self = shift;

  $self->request->environment->data_manager;
}

sub _make_rss {
  my ($self, $diary_day_list) = @_;

  # TODO refactoring with RssPage.pm
  my @items;

  for my $day ($diary_day_list->list) {
    for my $article ($day->article_list) {
      my $article_title = $article->title;
      my $date = $article->last_modified;
      my $link = $self->request->query->url . '/d/' . $article->id . '.html';
      my $description = $article->text_from_html;
      my $subject = $article_title;
      my $title = $article_title . ' (' . $article->date . ')';
      push (@items, [$title, $date, $link, $description, $subject]);
    }
  }

  my $result = '';
  $result .= qq(<?xml version="1.0" encoding="UTF-8" ?>\n);

  $result .= $self->_open_rdf;

  my $about_uri = $self->request->query->url;
  my $link_uri = $about_uri;
  my $channel_title = $self->_get_properties('dikicker.name');
  my $channel_description =
    $self->_get_properties('param.RssPage.channel.description');
  unless ($channel_description) {
    $channel_description = $channel_title;
  }

  $channel_title = WiKicker::HTML::escape($channel_title);
  $channel_description = WiKicker::HTML::escape($channel_description);

  $result .= <<EOS;
<channel rdf:about="$about_uri">
<title>$channel_title</title>
<link>$link_uri</link>
<description>$channel_description</description>
EOS

  $result .= qq(<items>\n<rdf:Seq>\n);
  for my $item (@items) {
    my $resource = WiKicker::HTML::escape_attribute($item->[2]);
    $result .= qq(<rdf:li rdf:resource="$resource" />\n);
  }

  $result .= qq(</rdf:Seq>\n</items>\n</channel>\n);

  for my $item (@items) {
    my $about = WiKicker::HTML::escape_attribute($item->[2]);
    my $title = WiKicker::HTML::escape($item->[0]);
    my $link = WiKicker::HTML::escape($item->[2]);
    my $description = WiKicker::HTML::escape($item->[3]);
    my $date = $item->[1];

    my $date_str = WiKicker::Time::gmtime_to_W3CDTF_UTC($date);

    $result .=<<EOS;
<item rdf:about="$about">
  <title>$title</title>
  <link>$link</link>
  <description>$description</description>
  <dc:date>$date_str</dc:date>
</item>
EOS
  }

  $result .= qq(</rdf:RDF>\n);

  $self->request->attribute('rss', $result);

  return 'rss';
}

sub _open_rdf {
  return <<EOS;
<rdf:RDF
  xmlns="http://purl.org/rss/1.0/"
  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
  xmlns:dc="http://purl.org/dc/elements/1.1/">
EOS
}


1;
