#
# WiKicker
#
#  Diaries.pm
#
#
# Copyright (C) 2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Diaries.pm,v 1.1 2005/03/26 13:30:39 naney Exp $
#
#

package WiKicker::DikiCGI::View::HTML::Diaries;

use 5.005;
use strict;

use WiKicker::DikiCGI::View::HTML;

use vars qw(@ISA);

BEGIN {
  @ISA = qw(WiKicker::DikiCGI::View::HTML);
}

sub title {
  my $self = shift;

  $self->request->configuration->properties->get('dikicker.name');
}

sub sub_title { '' }

sub make_html_body {
  my $self = shift;

  my $response = $self->response;

  my $top_url = $self->request->url;
  my $sub_title = $self->sub_title;
  my $title = WiKicker::HTML::escape($self->title);
  $response->add_content('<div class="main">');
  $response->add_content('<div class="adminmenu">');
  $response->add_content(qq(<span class="adminmenu"><a href="$top_url">トップ(最新)</a></span>));
  $response->add_content('</div>');
  $response->add_content("<h1>$title</h1>");
  if ($sub_title) {
    $sub_title = WiKicker::HTML::escape($sub_title);
    $response->add_content("<div>$sub_title</div>\n");
  }
  $self->keyword_articles;
  $self->make_diary_days;
  $self->make_calender;
}


sub make_html_head_title {
  my $self = shift;

  return $self->_make_html_head_title_no_article_title;
}

sub _make_html_head_title_with_article_title {
  my $self = shift;

  my $title = $self->request->attribute('diaries')->html_title;
  my $name = $self->request->configuration->properties->get('dikicker.name');
  return WiKicker::HTML::escape($name . ': ' . $title);
}

sub _make_html_head_title_no_article_title {
  my $self = shift;

  my $name = $self->request->configuration->properties->get('dikicker.name');
  return WiKicker::HTML::escape($name);
}


sub keyword_articles {
}

sub make_diary_days {
  my $self = shift;

  my $diary_day_list = $self->request->attribute('diaries');

  if ($diary_day_list->size == 0) {
    $self->response->add_content('no diary');
    return;
  }

  for my $day ($diary_day_list->list) {
    $self->make_diary_day($day);
  }
}


sub make_diary_day {
  my ($self, $diary_day) = @_;

  $self->response->add_content($diary_day->to_html($self->request->url));
}

sub make_calender {
  my $self = shift;

  my $ten_day_list = $self->request->environment->data_manager->ten_day_list;

  return if @$ten_day_list == 0;

  my $div = '<div class="calender">';

  my %year;
  for my $ten_day (@{$ten_day_list}) {
    $ten_day =~ /^(\d\d\d\d)/;
    $year{$1}->{$ten_day}++;
  }

  my $url = $self->request->url;
  for my $year (sort keys %year) {
    $div .= qq(<div class="year">$year|);
    my $count = 0;
    for my $ten_day (sort keys %{$year{$year}}) {
      if ($count >= 18) {
        $div .= qq(</div><div class="year">$year|);
        $count = 0;
      }
      $ten_day =~ /^\d\d\d\d-(\d\d)-(.)/;
      $div .= qq(<a href="$url/d/$ten_day.html">$1$2</a>|);
      $count++;
    }
    $div .= '</div>';
  }
  $div .= '</div>';
  $self->response->add_content($div);
}

1;
