#
# WiKicker
#
#  File.pm
#
#
# Copyright (C) 2003,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: File.pm,v 1.2 2005/05/07 04:44:22 naney Exp $
#
#

package WiKicker::File;

use 5.005;
use strict;

BEGIN {
  if ($^O =~ /Win32/) {
    eval "use MIME::Base32 qw(RFC)";
    die $@ if $@;

    *key_to_file_name = \&key_to_file_name_base32;
    *file_name_to_key = \&file_name_to_key_base32;
  }
  else {
    eval "use MIME::Base64";
    die $@ if $@;

    *key_to_file_name = \&key_to_file_name_base64;
    *file_name_to_key = \&file_name_to_key_base64;
  }

  *key_to_file_name_base32 = \&MIME::Base32::encode;
  *file_name_to_key_base32 = \&MIME::Base32::decode;
}

# use [A-Za-z0-9+.=]
# Oh my god! '=' is reserved within a path segment (RFC2396 3.3)
sub key_to_file_name_base64 {
  my $key = shift;

  my $name = MIME::Base64::encode_base64($key, '');
  chomp $name;
  $name =~ tr|/|.|;
  return $name;
}

sub file_name_to_key_base64 {
  my $file_name = shift;
  $file_name =~ tr|.|/|;
  return MIME::Base64::decode_base64($file_name);
}

# # use [A-Za-z0-9+.-]
# sub key_to_pchar_name {
#   my $key = shift;

#   my $name = encode_base64($key, '');
#   chomp $name;
#   $name =~ tr|/=|.-|;
#   return $name;
# }

# sub pchar_name_to_key {
#   my $file_name = shift;
#   $file_name =~ tr|.-|/=|;
#   return decode_base64($file_name);
# }

1;
