#
# WiKicker
#
#  Verify.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Verify.pm,v 1.2 2004/05/01 06:27:26 naney Exp $
#
#

package WiKicker::GnuPG::Verify;

use strict;

use vars qw($HAVE_GnuPG);
use IO::Handle;

BEGIN {
  eval "use GnuPG::Interface";
  if ($@) {
    $HAVE_GnuPG = 1;
  }
  else {
    $HAVE_GnuPG = 0;
  }
}


sub verify {
  my ($gpg, $gpg_home, $input_handle, $output_filename, $tmp_filename) = @_;

  my $gnupg = GnuPG::Interface->new();
  $gnupg->call($gpg);
  $gnupg->options->homedir($gpg_home);
  $gnupg->options->meta_interactive( 0 );

  my ( $input_fh, $output_fh, $error_fh, $status_fh )
    = (
       IO::Handle->new(),
       IO::Handle->new(),
       IO::Handle->new(),
       IO::Handle->new() );

  my $handles = GnuPG::Handles->new(
                                    stdin   => $input_fh,
                                    stdout  => $output_fh,
                                    stderr  => $error_fh,
                                    status  => $status_fh );


  $gnupg->decrypt( handles => $handles );

  my $buffer;
  while (read($input_handle, $buffer, 1024)) {
    print $input_fh $buffer;
  }
  close $input_fh;

  my ($error_output, $status_info);
  open(OUT, ">$tmp_filename") || die;
  while (read($output_fh, $buffer, 1024)) {
    print OUT $buffer;
  }
  close(OUT);
  {
    local $/ = undef;
    $error_output = <$error_fh>;
    $status_info = <$status_fh>;
  }

  wait;

  my $exit_code = $?;

  my $result = "exit code: $exit_code\nstderr:\n$error_output\nstatus_info:\n$status_info\n";
  if ($exit_code == 0) {
    rename($tmp_filename, $output_filename) || warn $!;
  }
  else {
    unlink($tmp_filename) || warn $!;
  }
  return ($exit_code, $result);
}

1;
