#
# WiKicker
#
#  Referer.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Referer.pm,v 1.6 2004/01/29 14:35:20 naney Exp $
#
#

package WiKicker::HTTP::Referer;

use strict;
use URI;
use WiKicker::HTML;
use WiKicker::URI;
use WiKicker::UTF8;

use vars qw(@search_engine_list);

BEGIN {
  @search_engine_list =
    (
     [qr!^http://www.google.co.jp/(search|ie)\?!, 'google.co.jp', \&google_query],
     [qr!^http://www.google.com/(search|custom)\?!, 'google.com', \&google_query],
     [qr!^http://google.com./search\?!, 'google.com', \&google_query],
     [qr|^http://www.google.co.kr/search\?|, 'google.co.kr', \&google_query],
     # yahoo
     [qr|^http://(web)?search.yahoo.co.jp/bin/query\?|, 'yahoo.co.jp', \&yahoo_query],
     [qr|^http://search.yahoo.co.jp/bin/search\?|, 'yahoo.co.jp', \&yahoo_query],
     [qr|^http://tw.websearch.yahoo.com/bin/search/web_kimo\?|, 'tw.websearch.yahoo.com', \&yahoo_query],

     # msn
     [qr|^http://search.msn.co.jp/results.aspx?\?|, 'msn.co.jp', \&msn_query],
     [qr|^http://search.msn.co.jp/spresults.aspx\?|, 'msn.co.jp', \&msn_query],

     # excite
     [qr|^http://www.excite.co.jp/search.gw\?|, 'excite.co.jp', \&excite_query],
     [qr|^http://dion.excite.co.jp/search.gw\?|, 'dion.excite.co.jp', \&excite_query],
     [qr|^http://home.excite.co.jp/search.gw\?|, 'home.excite.co.jp', \&excite_query],
     [qr|^http://odn.excite.co.jp/search.gw\?|, 'odn.excite.co.jp', \&excite_query],
     [qr|^http://so-net.excite.co.jp/search.gw\?|, 'so-net.excite.co.jp', \&excite_query],
     [qr|^http://apple.excite.co.jp/search.gw\?|, 'apple.excite.co.jp', \&excite_query],
     [qr|^http://dion.excite.co.jp/search.gw\?|, 'dion.excite.co.jp', \&excite_query],

     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search-st(_lp)?\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search_lp\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search_bl_top\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search-ie_lp\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search-tb12\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search-tb20b\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search-tb25\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search2-co\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search2-(co|b)\?!,'biglobe', \&biglobe_query],
     [qr!^http://cgi.search.biglobe.ne.jp/cgi-bin/search-ie\?!,'biglobe', \&biglobe_query],
     [qr!^http://search.goo.ne.jp/web.jsp\?!,'goo', \&goo_query],
     [qr!^http://ocnsearch.goo.ne.jp/ocn.jsp\?!,'goo', \&goo_query],
     [qr!^http://search.nifty.com/cgi-bin/search.cgi\?!,'nifty.com', \&nifty_query],
     [qr!^http://search.fresheye.com/\?!,'fresheye', \&fresheye_query],
     [qr!^http://www.infoseek.co.jp/OTitles\?!,'infoseek', \&infoseek_query],
     [qr!^http://search.jp.aol.com/webdir.adp\?!,'jp.aol.com', \&aol_query],
     [qr!^http://search.jp.aol.com/advhandler.adp\?!,'jp.aol.com', \&aol_query],
     [qr!^http://search.jword.jp/main.phtml\?!,'jword', \&jword_query],
     [qr!^http://search.naver.co.jp/search.naver\?!,'NAVER', \&naver_query],
    );
}

sub new {
  my ($class, $referer) = @_;

  my $self = bless {}, $class;
  if (defined $referer) {
    $self->{referer} = $referer;
  }
  else {
    $self->{referer} = $ENV{HTTP_REFERER};
  }

  return $self;
}

sub is_search_engine_result_page {
  my $self = shift;

  unless (defined $self->{referer}) {
    return 0;
  }

  for my $info (@search_engine_list) {
    my $pattern = $info->[0];
    return 1 if $self->{referer} =~ $pattern;
  }
  return 0;
}

sub get_search_query {
  my $self = shift;

  unless (defined $self->{referer}) {
    return (undef, undef);
  }

  for my $info (@search_engine_list) {
    my $pattern = $info->[0];
    if ($self->{referer} =~ $pattern) {
      my $query = &{$info->[2]}($self->{referer});
      if (defined $query) {
        return ($info->[1], $query);
      }
      return (undef, undef);
    }
  }
  return (undef, undef);
}

sub make_html_link {
  my $self = shift;

  if (!(defined $self->{referer}) || $self->{referer} eq '') {
    return '';
  }
  if ($self->{referer} =~ /^(java-script|vbscript|about):/i) {
    return WiKicker::HTML::escape($self->{referer});
  }

  my $url = WiKicker::HTML::escape_attribute($self->{referer});
  my $link_text;

  my ($type, $query) = $self->get_search_query;
  if (defined $type) {
    $link_text = WiKicker::HTML::escape("$type: $query");
  }
  else {
    $link_text = WiKicker::HTML::escape(WiKicker::URI::short_http_url($self->{referer}, 40));
  }
  return qq(<a href="$url">$link_text</a>);
}

sub google_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{q}) {
    return WiKicker::UTF8::decode_from_unknown($param{q});
  }
  else {
    return undef;
  }
}

sub yahoo_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{p}) {
    return WiKicker::UTF8::decode_from_unknown($param{p});
  }
  return undef;
}

sub msn_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{q}) {
    return WiKicker::UTF8::decode_from_unknown($param{q});
  }
  return undef;
}

sub excite_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{search}) {
    return WiKicker::UTF8::decode_from_unknown($param{search});
  }
  return undef;
}

sub biglobe_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{q}) {
    return WiKicker::UTF8::decode_from_unknown($param{q});
  }
  return undef;
}

sub goo_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{MT}) {
    return WiKicker::UTF8::decode_from_unknown($param{MT});
  }
  return undef;
}

sub nifty_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{Text}) {
    return WiKicker::UTF8::decode_from_unknown($param{Text});
  }
  return undef;
}

sub fresheye_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{kw}) {
    return WiKicker::UTF8::decode_from_unknown($param{kw});
  }
  return undef;
}

sub infoseek_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{qt}) {
    return WiKicker::UTF8::decode_from_unknown($param{qt});
  }
  return undef;
}


sub aol_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{query}) {
    return WiKicker::UTF8::decode_from_unknown($param{query});
  }
  return undef;
}

sub jword_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{name}) {
    return WiKicker::UTF8::decode_from_unknown($param{name});
  }
  return undef;
}

sub naver_query {
  my $url = shift;
  
  my $uri_obj = URI->new($url);
  my %param = $uri_obj->query_form;
  if (exists $param{query}) {
    return WiKicker::UTF8::decode_from_unknown($param{query});
  }
  return undef;
}

1;
