#
# WiKicker
#
#  Wrapper.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Wrapper.pm,v 1.3 2004/08/03 12:55:52 naney Exp $
#
#
package WiKicker::Kagemai::Wrapper;

use 5.005;
use strict;
use WiKicker::App::MarkUpAsHtml;
use WiKicker::UTF8;

#
# use WiKicker::Kagemai::Wrapper;
# WiKicker::Kagemai::Wrapper->new(
#  kagemai_script => '/var/www/cgi-bin/kagemai/guest-original.cgi',
#  wikicker_properties => '/var/www/cgi-bin/wiki.properties',
#  wikicker_url        => 'http://www.example/wiki'
#  begin_string        => '<!-- begin message-body -->',
#  end_string          => '<!-- end message-body -->'
#)
#  ->run;
#

sub new {
  my ($class, %options) = @_;

  die 'No kagemai_script option' unless $options{kagemai_script};
  die 'No wikicker_properties option' unless $options{wikicker_properties};
  die 'No wikicker_url option' unless $options{wikicker_url};
  die 'No begin_string option' unless $options{begin_string};
  die 'No end_string option' unless $options{end_string};

  my $self = bless {}, $class;

  for my $key (keys %options) {
    $self->{$key} = $options{$key};
  }

  return $self;
}

sub run {
  my $self = shift;

  my $current_script_filename = $ENV{SCRIPT_FILENAME};
  if ($current_script_filename &&
      $current_script_filename eq $self->{kagemai_script}) {
    die "recursive call: $current_script_filename";
  }
  $ENV{SCRIPT_FILENAME} = $self->{kagemai_script};
  $| = 1;
  $self->_wrapper;
  $ENV{SCRIPT_FILENAME} = $current_script_filename;
}

sub _wrapper {
  my $self = shift;

  open(CGI, $self->{kagemai_script} . ' |') ||
    die q(Can't open ) . $self->{kagemai_script} . ": $!";

  my $is_html = 0;
  while (<CGI>) {
    my $line = $_;
    if ($line =~ m|^Content-Length:|i) {
      next;
    }
    print $line;
    if ($line =~ m|^Content-Type:\s+text/html|i) {
      $is_html = 1;
    }
    if ($line eq "\015\012") {
      last;
    }
  }

  unless ($is_html) {
    while (<CGI>) {
      print $_;
    }
    close (CGI);
    return;
  }

  my $begin_string = $self->{begin_string};
  my $end_string = $self->{end_string};
  my $in_body = 0;
  my @in_body_text;

  while (<CGI>) {
    my $line = $_;
    if ($line =~ /\Q$begin_string\E/) {
      $in_body = 1;
      next;
    }
    if ($line =~ /\Q$end_string\E/) {
      $in_body = 0;
      print $self->_replace_lines(@in_body_text);
      @in_body_text = ();
      next;
    }

    if ($in_body) {
      push(@in_body_text, $line);
    }
    else {
      print $line;
    }
  }
  if (@in_body_text) {
    print $self->_replace_lines(@in_body_text);
  }
  close (CGI);
}

sub _markup_tool {
  my $self = shift;

  unless (exists $self->{_markup_tool}) {
    $self->{_markup_tool} = WiKicker::App::MarkUpAsHtml
      ->new($self->{wikicker_properties},
            $self->{wikicker_url},
            1);
  }
  return $self->{_markup_tool}
}

sub _markup {
  my ($self, $text) = @_;

  $text = WiKicker::UTF8::decode('euc', $text);
  my $result = $self->_markup_tool->markup($text);
  $result = WiKicker::UTF8::encode('euc', $text);
  return $result;
}

sub _replace_lines {
  my ($self, @lines) = @_;

  my $result = '';
  # first line
  if (@lines) {
    my $first_line = shift(@lines);
    $first_line =~ s/^\s*<pre>//;
    unshift(@lines, $first_line);
  }
  if (@lines) {
    my $last_line = pop(@lines);
    $last_line =~ s|\s*</pre>\s*$||;
    push(@lines, $last_line);
  }

  my $in_pre = 0;
  my $buffer = '';
  for my $line (@lines) {
    if ($line =~ /^$/) {
      if ($in_pre) {
        $result .= $line;
      }
      elsif (length($buffer)) {
        $result .= "<p>\n";
        $result .= $self->_replace($buffer);
        $result .= "\n</p>\n";
        $buffer = '';
      }
    }
    elsif ($line =~ /^(\s|&gt;)/) {
      unless ($in_pre) {
        if (length($buffer)) {
          $result .= "<p>\n";
          $result .= $self->_replace($buffer);
          $result .= "\n</p>\n";
          $buffer = '';
        }
        $in_pre = 1;
        $result .= '<pre>';
      }
      $result .= $self->_replace($line);
    }
    else {
      if ($in_pre) {
        $result .= "</pre>\n";
        $in_pre = 0;
      }
      chomp $line;
      $buffer .= $line;
    }
  }
  if ($in_pre) {
    $result .= "</pre>\n";
  }
  elsif (length($buffer)) {
    $result .= "<p>\n";
    $result .= $self->_replace($buffer);
    $result .= "\n</p>\n";
  }

  return $result;
}

sub _replace {
  my ($self, $str) = @_;

  my $result;
  my $length = 0;
  while ($str =~
         m/\G([\x00-\xFF]*?)        #skip
           (
              (<a\s.*?<\/a>)            # anchor
            | (<\/?.*?>)               # tag
           )
         /ogx) {
    my $skip = $1;
    my $special = $2;
    if (length $skip) {
      $length += length($skip);
      $result .= $self->_markup($skip);
    }
    if (length $special) {
      $length += length($special);
      $result .= $special;
    }
  }
  if (length($str) > $length) {
    $result .= $self->_markup(substr($str, $length));
  }

  return $result;
}

1;
