#
# WiKicker
#
#  LoadAverage.pm
#
#
# Copyright (C) 2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: LoadAverage.pm,v 1.3 2005/04/04 13:52:13 naney Exp $
#
#

package WiKicker::LoadAverage;

use 5.005;
use strict;

use vars qw($CACHE $CACHE_TIME);
BEGIN {
  $CACHE_TIME = 0;
}

sub new {
  my $class = shift;

  return bless {}, $class;
}

sub _initialize_load_average {
  my $self = shift;

  if (exists $self->{load_average_initialized}) {
    return;
  }

  if ($self->_initialize_load_average_by_proc) {
    return;
  }


  my $now = time;
  if ($CACHE_TIME > 0 && $now < $CACHE_TIME + 5) {
    $self->{load_average_string} = $CACHE;
  }
  else {
    # @TODO Get load average from system on Win32.
    my $load_average_result = 'load average: 1.0, 1.0, 1.0';
    unless ($^O =~ /Win32/) {
      $load_average_result = `uptime`;
    }
    $load_average_result =~ /(load.*)$/;
    $self->{load_average_string} = $1 || '';
    $CACHE = $self->{load_average_string};
    $CACHE_TIME = $now;
  }


  if ($self->{load_average_string}
      =~ /(\d+(?:\.\d+)?),?\s+(\d+(?:\.\d+)?),?\s+(\d+(?:\.\d+)?)/) {
    $self->{load_average_1} = $1;
    $self->{load_average_5} = $2;
    $self->{load_average_15} = $3;
  }
  else {
    $self->{load_average_1} = -1;
    $self->{load_average_5} = -1;
    $self->{load_average_15} = -1;
  }

  $self->{load_average_initialized} = 1;
}

# Linux
sub _initialize_load_average_by_proc {
  my $self = shift;

  return 0 unless -e '/proc/loadavg';
  open(LOADAVG, '/proc/loadavg') || return 0;
  {
    local $/;
    my $loadavg = <LOADAVG>;
    close LOADAVG;
    $loadavg =~ /^(\d+\.\d+)\s(\d+\.\d+)\s(\d+\.\d+)/;
    if (defined($1) && defined($2) && defined($3)) {
      $self->{load_average_1} = $1;
      $self->{load_average_5} = $2;
      $self->{load_average_15} = $3;
      $self->{load_average_string} = "load averages: $1, $2, $3";
      $self->{load_average_initialized} = 1;
      return 1;
    }
  }
  return 0;
}

sub load_average_string {
  my $self = shift;

  $self->_initialize_load_average;
  return $self->{load_average_string};
}

sub load_average_1 {
  my $self = shift;

  $self->_initialize_load_average;
  return $self->{load_average_1};
}

1;
