#
# WiKicker
#
#  Mailer.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Mailer.pm,v 1.3 2005/04/15 12:55:53 naney Exp $
#
#

package WiKicker::Mailer;

use 5.005;
use strict;

use Socket;
#use MIME::Words;
use MIME::Base64;
use WiKicker::UTF8;

sub new {
  my ($class, $mail_program) = @_;
  my $self = bless { mail_program => $mail_program }, $class;

  unless ($self->{mail_program}) {
    $self->{mail_program} = $self->detect_mail_program;
  }
  return $self;
}

sub detect_mail_program {
  my $self = shift;

  if (-x '/usr/lib/sendmail') {
    return '/usr/lib/sendmail -t';
  }
  if (-x '/usr/sbin/sendmail') {
    return '/usr/sbin/sendmail -t';
  }

  return '';
}

sub make_message {
  my ($self, $forum_name, $to, $page, $mode, $page_body, $user) = @_;
  my $user_name = 'unknown';
  my $id = 'unknown';
  if ($user) {
    $user_name = $user->name;
    $id = $user->id;
  }
  #my $title = encode_utf8_words($page . ' - ' . $user_name);
  my $subject = encode_utf8_words("[$forum_name] $mode - $page - $user_name");

  my $remote_addr = $ENV{REMOTE_ADDR} || '';
  my $remote_host = $ENV{REMOTE_HOST} || '';

  if ((! $remote_host) && $remote_addr) {
    $remote_host = gethostbyaddr(inet_aton($remote_addr), AF_INET);
  }

  my $message = <<"EOD";
To: $to
From: $to
Subject: $subject
MIME-Version: 1.0
Content-Type: text/plain; charset=UTF-8
Content-Transfer-Encoding: 8bit

--------
MODE = $mode
REMOTE_ADDR = $remote_addr
REMOTE_HOST = $remote_host
USER        = $user_name
ID          = $id
--------
$page
--------
$page_body
--------
EOD

  return $message;
}

sub send {
  my ($self, $forum_name, $to, $page, $mode, $page_body, $user) = @_;
  my $message = $self->make_message($forum_name,
                                    $to, $page, $mode, $page_body, $user);
  my $sendmail = $self->{mail_program};
  return if $sendmail eq '';

  open(MAIL, "| $sendmail") || die "Can't open: $!";
  print MAIL $message;
  close(MAIL);
}

use vars qw($NONPRINT);
BEGIN { $NONPRINT = "\\x00-\\x1F\\x7F-\\xFF"; } # MIME::Words

# RFC 2047 (RFC1522)
sub encode_utf8_words {
  my $text = shift;

  if ($text eq '') {
    return '';
  }

  if ($text !~ /[$NONPRINT]/o) {
    return $text;
  }
#   my @chars = WiKicker::UTF8::split_into_array($text);
#   my $result = '=?UTF-8?Q?';
#   my $encoded_word_length = 10;
#   for my $c (@chars) {
#     my $encoded = $c;
#     $encoded =~ s{([_\?\=$NONPRINT])}{'=' . uc(unpack('H2', $1))}eg;
#     $encoded =~ s/ /_/g;
#     if ($encoded_word_length + length($encoded) + 2 > 75) {
#       $result .= qq(?=\r\n =?UTF-8?Q?);
#       $encoded_word_length = 10;
#     }
#     $encoded_word_length += length($encoded);
#     $result .= $encoded;
#   }
#   $result .= '?=';
#  return $result;


#   my $size = length($text);

#   my @result;
#   for (my $i = 0; $i < $size; $i += 45) {
#     my $splitted = substr($text, $i, 45);
#     my $encoded = '=?UTF-8?B?' . MIME::Base64::encode_base64($splitted, '') . '?=';
#     push(@result, $encoded);
#   }
#   return join("\r\n ", @result);

  my @chars = WiKicker::UTF8::split_into_array($text);
  my @result;
  while (@chars) {
    my $substring = '';
    my $length = 0;
    my $c;
    while (defined($c = shift(@chars))) {
      $length += length($c);
      if ($length > 45) {
        unshift(@chars, $c);
        last;
      }
      $substring .= $c;
    }
    if ($substring ne '') {
      push(@result, '=?UTF-8?B?' . MIME::Base64::encode_base64($substring, '') . '?=');
    }
  }
  return join("\n ", @result); # CR is added in make_message

#   use Jcode;
#     my @chars = WiKicker::UTF8::split_into_array($text);
#   @chars = map { my $c = $_; Jcode::convert(\$c, 'jis', 'utf8'); $c } @chars;
#    my @result;
#    while (@chars) {
#      my $substring = '';
#      my $length = 0;
#      my $c;
#      while (defined($c = shift(@chars))) {
#        $length += length($c);
#        if ($length > 45) {
#          unshift(@chars, $c);
#          last;
#        }
#        $substring .= $c;
#      }
#      if ($substring ne '') {
#        push(@result, '=?iso-2022-jp?B?' . MIME::Base64::encode_base64($substring, '') . '?=');
#      }
#    }
#    return join("\r\n ", @result);
}

1;
