#
# WiKicker
#
#  MultiSetWordExtractor.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: MultiSetWordExtractor.pm,v 1.1 2004/02/08 13:14:41 naney Exp $
#
#

package WiKicker::Text::MultiSetWordExtractor;

use strict;
use WiKicker::Text::WordExtractor;

sub new {
  my $class = shift;

  my $self = bless {}, $class;

  return $self;
}

sub set_words_list_list {
  my ($self, $words_list_list_ref) = @_;

  for my $list (@$words_list_list_ref) {
    my $word_extractor = WiKicker::Text::WordExtractor->new;
    $word_extractor->set_words($list, 1);

    push (@{$self->{extractor}}, $word_extractor);
  }
}

sub split_into_words {
  my ($self, $text) = @_;

  my $result = [ [0, $text] ];

  my $extractor_id = 0;
  for my $extractor (@{$self->{extractor}}) {
    $extractor_id++;
    
    my $size = scalar(@$result);
    my $i = 0;
    while ($i < $size) {
      if ($result->[$i]->[0] != 0) {
        $i++;
        next;
      }
      my $fragment_text = $result->[$i]->[1];

      my $fragment_splited = $extractor->split_into_words($fragment_text);
      my $fragment_size = scalar(@$fragment_splited);
      for (my $j = 0; $j < $fragment_size; $j++) {
        if ($fragment_splited->[$j]->[0] != 0) {
          $fragment_splited->[$j]->[0] = $extractor_id;
        }
      }
      splice(@$result, $i, 1, @$fragment_splited);
      $size = $size - 1 + $fragment_size;
      $i += $fragment_size;
    }
  }
  return $result;
}

1;
