#
# WiKicker
#
#  OnDemandPageNameTrie.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: OnDemandPageNameTrie.pm,v 1.4 2004/06/21 15:40:20 naney Exp $
#
#

package WiKicker::Tree::OnDemandPageNameTrie;

use strict;
use WiKicker::UTF8;
use WiKicker::Tree::PageNameTrie;

sub new {
  my $class = shift;

  my $self = bless {}, $class;

  return $self;
}

sub add {
  my ($self, $page_name, $enable_wiki_name) = @_;

  # It must be recognized by InlineParser.
  if ((!$enable_wiki_name) && $page_name =~ m|^[A-Z][a-z]+(?:[A-Z][a-z]+)+(?:(?:/[A-Z][a-z]+(?:[A-Z][a-z]+)+)+)*$|o) {
    return;
  }

  my $last_c = WiKicker::UTF8::last_char($page_name);
  push(@{$self->{$last_c}}, \$page_name);
}

use vars qw($empty_array);
BEGIN {
  $empty_array = [];
}
sub lookup {
  my ($array_ref, $first, $next, $self) = @_;

  my $c = $array_ref->[$next];
  unless (exists $self->{$c}) {
    #return [];
    return $empty_array;
  }
  my $trie;
  if (ref $self->{$c} eq 'ARRAY') {
    $trie = WiKicker::Tree::PageNameTrie->new;
    for my $page_name_ref (@{$self->{$c}}) {
      my $c_list_ref = WiKicker::UTF8::split_into_array_ref($$page_name_ref);
      if (@$c_list_ref >= 2) {
        $trie->_add($c_list_ref, $page_name_ref);
      }
    }
    $trie->mark_abbrev;
    $self->{$c} = $trie;
  }
  else {
    $trie = $self->{$c};
  }
  return
    WiKicker::Tree::PageNameTrie::lookup($array_ref, $first, $next, $trie);
}

1;
