#
# WiKicker
#
#  Datetime.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Datetime.pm,v 1.5 2004/05/30 04:09:37 naney Exp $
#
#

package WiKicker::WRI::Datetime;

use strict;
use WiKicker::WRI::RWRN;
use WiKicker::Time;

use vars qw(@ISA);
@ISA = qw(WiKicker::WRI::RWRN);

sub gmt_time {
  WiKicker::Time::W3CDTF_to_gmtime(shift->rest);
}

sub to_html {
  my ($self, $content, $env) = @_;

  my $zone = 'GMT';
  if ($env->user) {
    $zone = $env->user->timezone || 'GMT';
  }

  my $time_str;
  if ($self->rest =~ /^(\d\d\d\d)-(\d\d)-(\d\d)$/) {
    if ($env->localizer) {
      $time_str = $env->localizer->language_handle->date_only_html_str($1, $2, $3, $zone);
    }
    else {
      $time_str = WiKicker::Time::japanese_date_only_html_str($1, $2, $3, $zone);
    }
  }
  else {
    my $gmtime = $self->gmt_time;
    unless (defined $gmtime) {
      return WiKicker::HTML::escape($content || $self->to_string);
    }
    if ($gmtime < 0) {
      return WiKicker::HTML::escape($content || $self->to_string);
    }

    if ($env->localizer) {
      $time_str = $env->localizer->language_handle->date_html_str($gmtime, $zone);
    }
    else {
      $time_str = WiKicker::Time::japanese_date_html_str($gmtime, $zone);
    }
  }
  return qq(<span class="date">$time_str</span>);
}

sub to_subject_string {
  my ($self, $env) = @_;

  if ($self->rest =~ /^(\d\d\d\d)-(\d\d)-(\d\d)$/) {
    #return WiKicker::Time::japanese_date_only_str($1, $2, $3, 'GMT');
    return WiKicker::Time::english_date_only_str($1, $2, $3, 'GMT');
  }
  my $gmtime = $self->gmt_time;
  unless (defined $gmtime) {
    return WiKicker::HTML::escape($self->rest);
  }
  if ($gmtime < 0) {
    return WiKicker::HTML::escape($self->rest);
  }

  #return WiKicker::Time::japanese_date_str($gmtime, 'GMT');
  return WiKicker::Time::english_date_str($gmtime, 'GMT');
}

1;
