#
# WiKicker
#
#  Image.pm
#
#
# Copyright (C) 2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Flickr.pm,v 1.1 2005/02/26 07:44:15 naney Exp $
#
#

package WiKicker::WRI::Flickr;

use strict;
use Carp;
use WiKicker::WRI::RWRN;

use vars qw(@ISA);
@ISA = qw(WiKicker::WRI::RWRN);

# Flickr web address alias pert.
sub _alias {
  my $self = shift;

  $self->rest =~ /^([a-zA-Z0-9_-]+?):/;
  return $1;
}

sub _image_url {
  my $self = shift;

  $self->rest =~ /^[a-zA-Z0-9_-]+?:(.*)$/;
  my $url = $1;
  unless ($url =~ /^(java-script|vbscript|about):/i) {
    return $url;
  }
  return '';
}

sub get_link_url {
  my $self = shift;

  my $alias = $self->_alias;
  my $image_url = $self->_image_url;
  return '' unless length($alias) > 0 && length($image_url) > 0;
  $image_url =~ m|.+/(.*)$|;
  my $file_name = $1;
  $file_name =~ m|([^_]+?)_[^.]+\.jpg$|;
  return '' unless (length($1 || '') > 0);
  return "http://www.flickr.com/photos/$alias/$1/";
}

sub get_image_url {
  my $self = shift;

  my $alias = $self->_alias;
  my $image_url = $self->_image_url;
  return '' unless (length($alias) > 0 && length ($image_url) > 0);
  return $image_url;
}

sub _make_image_link_html {
  my ($self, $content, $image_url, $env) = @_;
  my $href = $self->get_link_url($env);

  my $alt_string = WiKicker::HTML::escape($content || $self->to_string);

  my $align = '';
  my $scheme = $self->scheme;
  if ($scheme eq 'l-flickr') {
    $align = qq( class="left");
  }
  elsif ($scheme eq 'r-flickr') {
    $align = qq( class="right");
  }

  if ($href) {
    return qq(<a href="$href"><img src="$image_url" alt="$alt_string"$align></a>);
  }
  else {
    return qq(<img src="$image_url" alt="$alt_string"$align>);
  }
}

sub to_subject_string {
  my ($self, $env) = @_;

  return '';
}

1;
