#
# WiKicker
#
#  Form.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Form.pm,v 1.1 2003/11/08 04:34:44 naney Exp $
#
#

package WiKicker::WRI::Form;

use strict;
use WiKicker::WRI::RWRN;

use vars qw(@ISA);
BEGIN {
  @ISA = qw(WiKicker::WRI::RWRN);
}

sub to_html {
  my ($self, $content, $env) = @_;

  if ($self->rest eq 'comment' || $self->rest eq 'rcomment') {
    $env->comment_number($env->comment_number + 1);
    return $self->_make_comment_form($env);
  }
  elsif ($self->rest eq 'long') {
    $env->comment_number($env->comment_number + 1);
    return $self->_make_long_form($env);
  }
  else {
    return '';
  }
}

sub to_subject_string {
  my ($self, $env) = @_;

  return '';
}

sub _user_name {
  my ($self, $env) = @_;
  
  if ($env->user && $env->user->name) {
    return WiKicker::HTML::escape_attribute($env->user->name);
  }
  return '';
}

sub _make_comment_form {
  my ($self, $env) = @_;

  if ($env->in_inline) {
    return '';
  }

  my $query = $env->query;
  my $result = $self->_make_start_form($env);
  $result .= qq(<div class="comment-form">\n  );
  $result .= $self->_make_hidden($env);
  $result .= qq(\n );
  $result .= 'あなたの名前: ';
  $result .= $self->_make_name_field($env);
  $result .= qq(<br>);
  $result .= qq(\n  );
  $result .= $query->textfield(-name => 'comment.message',
                               -size => '50',
                               -default => '',
                               -override => 1,
                               -style => 'width:100%',
                              );
  $result .= qq(\n  );
  $result .= qq(<br>);
  $result .= $query->submit(-name => 'submit',
                            -value => 'コメント追加');
  $result .= qq(\n</div>);
  $result .= $query->end_form;
  $result .= "\n";

  return $result;
}

sub _make_long_form {
  my ($self, $env) = @_;

  if ($env->in_inline) {
    return '';
  }

  my $query = $env->query;
  my $result = $self->_make_start_form($env);
  $result .= qq(<div class="comment-form">\n);
  $result .= $self->_make_hidden($env);
  $result .= qq(\n );
  $result .= 'あなたの名前: ';
  $result .= $self->_make_name_field($env);
  $result .= '<br>';
  $result .= $query->textarea(-name => 'comment.message',
                              -default => '',
                              -rows => 8,
                              -columns => 80,
                              -override => 1,
                              -style => 'width: 100%',
                             );
  $result .= '<br>';
  $result .= $query->submit(-name => 'submit',
                            -value => 'コメント追加');
  $result .= '</div>';
  $result .= $query->end_form;
  $result .= "\n";

  return $result;
}

sub _make_name_field {
  my ($self, $env) = @_;
  
  return $env->query->textfield(-name => 'comment.name',
                                -default => $self->_user_name($env),
                                -size => '20');
}

sub _make_name_hidden {
  my ($self, $env) = @_;

  return $env->query->hidden(-name => 'name',
                             -default => $env->page_name,
                             -override => 1);
}

sub _make_action_hidden {
  my ($self, $env) = @_;

  return $env->query->hidden(-name     => 'action',
                             -default  => 'comment',
                             -override => 1);
}

sub _make_lastmodified_hidden {
  my ($self, $env) = @_;
  return $env->query->hidden(-name => 'lastmodified',
                             -default => $env->db->last_modified($env->page_name));
}

sub _make_comment_number_hidden {
  my ($self, $env) = @_;
  
  return $env->query->hidden(-name => 'comment.number',
                             -default => $env->comment_number);
}
  
sub _form_charset_hidden {
  my ($self, $env) = @_;

  my $charset = $env->form_charset || 'UTF-8';

  return $env->query->hidden(-name => 'form.charset',
                             -default => $charset,
                             -override => 1);
}

sub _make_hidden {
  my ($self, $env) = @_;

  my $result = '';
  $result .= $self->_make_name_hidden($env) . qq(\n );
  $result .= $self->_make_action_hidden($env) . qq(\n );
  $result .= $self->_make_lastmodified_hidden($env) . qq(\n  );
  $result .= $self->_make_comment_number_hidden($env) . qq(\n  );
  $result .= $self->_form_charset_hidden($env);

  return $result;
}

sub _make_start_form {
  my ($self, $env) = @_;

  return $env->query->start_form(-method => 'POST',
                                 -action => $env->wikicker_url);
}

1;
