#
# WiKicker
#
#  SWRN.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: SWRN.pm,v 1.3 2005/03/20 10:44:26 naney Exp $
#
#

package WiKicker::WRI::SWRN;

use strict;
use WiKicker::WRI::NQWRN;

use vars qw(@ISA);
@ISA = qw(WiKicker::WRI::NQWRN);

sub new {
  my ($class, $original) = @_;

  my $self = bless {}, $class;

  $self->_set_original($original);

  return $self;
}

sub canonical {
  my $self = shift;

  if ($self->original =~ /^(isbn|ISBN)(.*)$/) {
    return WiKicker::WRI::URN->new('urn:isbn:' . $2,
                                   'urn',
                                   'isbn:' . $2);
  }
  if ($self->original =~ /^(rfc|RFC)(.*)$/) {
    return WiKicker::WRI::URN->new("urn:ietf:rfc:$2",
                                   'urn',
                                   "ietf:rfc:$2"
                                  );
  }
  return undef;
}

sub to_html {
  my ($self, $content, $env) = @_;

  return $self->_make_url_link_html($content, $env);
}

sub get_link_url {
  my ($self, $env) = @_;

  my $canonical = $self->canonical;
  return '' unless defined $canonical;
  return $canonical->get_link_url($env);
}

sub get_image_url {
  return '';
}

sub _is_isbn {
  my $self = shift;

  my $canonical = $self->canonical;
  if (defined $canonical) {
    if ($canonical->isa('WiKicker::WRI::URN') && $canonical->nid eq 'isbn') {
      return 'ISBN' . WiKicker::ISBN::hyphenation($canonical->nss);
    }
  }
  return '';
}

sub _make_url_link_html {
  my ($self, $content, $env) = @_;

  my $image_url = $self->get_image_url($env);
  if ($image_url) {
    return $self->_make_image_link_html($content, $image_url, $env);
  }
  else {
    my $url = WiKicker::HTML::escape_attribute($self->get_link_url($env));
    if ($url) {
      my $isbn = $self->_is_isbn;
      my $content_string;
      if (!($content) && $isbn) {
        $content_string = WiKicker::HTML::escape($isbn);
      }
      else {
        $content_string = WiKicker::HTML::escape($content || $self->to_string);
      }
      return qq(<a href="$url">$content_string</a>);
    }
    else {
      return WiKicker::HTML::escape($content || $self->to_string);
    }
  }
}

1;
