#
# WiKicker
#
#  IgnorePatternFile.pm
#
#
# Copyright (C) 2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: IgnorePatternFile.pm,v 1.1 2005/01/09 09:33:23 naney Exp $
#
#
package WiKicker::WikiCGI::IgnorePatternFile;

use 5.005;
use strict;
use Log::Log4perl;

sub new {
  my ($class, %option) = @_;

  my $self = bless {}, $class;

  if (exists $option{file}) {
    $self->_load_file($option{file});
  }

  return $self;
}

sub _load_file {
  my ($self, $file) = @_;

  return unless -e $file;

  unless (open(FILE, $file)) {
    Log::Log4perl->get_logger->error("Can't load file $file: $!");
    return;
  }

  my $line;
  while (defined($line = <FILE>)) {
    chomp $line;
    $line =~ s/^\s+//;
    $line =~ s/\s+$//;
    next if $line eq '';
    next if $line =~ /^#/;
    if ($line =~ /^qr/) {
      my $pattern = eval $line;
      if ($@) {
        push(@{$self->{pattern}}, qr/\Q$line\E/);
        next;
      }
      unless (ref($pattern) eq 'Regexp') {
        push(@{$self->{pattern}}, qr/\Q$line\E/);
        next;
      }
      push(@{$self->{pattern}}, $pattern);
      next;
    }
    push(@{$self->{pattern}}, qr/\Q$line\E/);
    next;
  }
  close(FILE);
}

sub match {
  my ($self, $text) = @_;

  for my $pattern (@{$self->{pattern}}) {
    return 1 if $text =~ $pattern;
  }

  return 0;
}

1;
