#
# WiKicker
#
#  ImageUploadPage.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: ImageUploadPage.pm,v 1.3 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::WikiCGI::Page::ImageUploadPage;

use 5.005;
use strict;
use WiKicker::WikiCGI::Page;
use WiKicker::GnuPG::Verify;

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiCGI::Page);

sub do_read {
  my ($self, $response) = @_;

  my $filename = $self->request->query->param('file');

  unless ($filename) {
    $self->db->shared_lock_pages;
    return 'imageuploadform';
  }

  $filename =~ /(\.png\.gpg|\.jpg\.gpg|\.jpeg\.gpg)$/;
  my $suffix = $1 || '';

  if ($suffix eq '') {
    #Invalid file name. Only .png.gpg / .jpg.gpg / .jpeg.gpg
    return 'error';
  }

  my $handle = $filename;
  my $size = -s $handle;
  my $max_size = 20;

  if ($size > $max_size*1024) {
    #"The file is too big ($size > $max_size KB)";
    return 'error';
  }

  # open file
  $suffix =~ s/\.gpg$//;
  my  $upload_name = time() . $$ . $suffix;

  my $gpg_command = $self->request->configuration->properties->get('program.gpg');
  my $gpg_home = $self->request->configuration->properties->get('gpg.home');

  my $data_dir = $self->request->configuration->properties->get('param.ImageUploadPage.datadir');

  my $tmp_dir = $self->request->configuration->properties->get('param.ImageUploadPage.tmpdir');

  my $data_url = $self->request->configuration->properties->get('param.ImageUploadPage.ImageBaseUrl');

  my ($status, $result)
    = WiKicker::GnuPG::Verify::verify(
                                      $gpg_command,
                                      $gpg_home,
                                      $handle,
                                      $data_dir . $upload_name,
                                      $tmp_dir . $upload_name);

  warn $result;

  $self->mail('ImageUploadPage', 'upload', "$upload_name\n$result");


  if ($status) {
    return 'error';
  }

  #my $cookie = new CGI::Cookie(-name => 'uploadimg',
  #                           -value => { 'url' => $data_url . $upload_name },
  #                           -expires => '+1d',
  #                           -domain => '.naney.org',
  #                           -path => '/wiki');

  $self->request->attribute('imageurl', $data_url . $upload_name);
  return 'imageuploadsuccess';
}

1;
