#
# WiKicker
#
#  HTML.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: HTML.pm,v 1.35 2004/07/03 06:43:10 naney Exp $
#
#

package WiKicker::WikiCGI::View::HTML;

use 5.005;
use strict;
use WiKicker;
use WiKicker::CGI::View::HTML;
use WiKicker::HTML;
use WiKicker::HTML::LinkMaker;
use WiKicker::URI;
use WiKicker::PageName;
use WiKicker::WRI::Resolver;
use WiKicker::Time;
use WiKicker::WikiPage::WikiPageHtmlFragmentVisitor;
use WiKicker::WikiPage::tDiaryFragmentVisitor;
use WiKicker::WikiPage::Env;
use AutoLoader 'AUTOLOAD';

use vars qw(@ISA);
BEGIN {
  @ISA = qw(WiKicker::CGI::View::HTML);
}

sub db {
  shift->request->environment->db;
}

sub link_maker {
  my $self = shift;

  if ($self->{link_maker}) {
    return $self->{link_maker};
  }
  else {
    my $link_maker = $self->{link_maker}
      = WiKicker::HTML::LinkMaker
        ->new(script_url => $self->request->url,
              db => $self->db,
              configuration => $self->request->configuration,
              timezone => $self->request->user ? ($self->request->user->timezone || 'GMT') : 'GMT',
              localizer => $self->request,
              page_name_list => $self->db->page_name_list);
  }
  return $self->{link_maker}
}

sub _link {
  my ($rel, $uri) = @_;

  my $escaped = WiKicker::HTML::escape_attribute($uri);
  return qq(  <link rel="$rel" href="$escaped">);
}

sub make_link_element_list {
  my $self = shift;

  my $name = $self->request->name;

  my @list;

  push(@list, _link('start', $self->front_page_url));
  push(@list, _link('index', $self->index_page_url));
  push(@list, _link('search', $self->request->url . '/SearchPage.html'));

  my @ancestors = $self->db->page_name_list->ancestors_page_name_list($name);
  if (@ancestors) {
    push(@list, _link('up',
                      WiKicker::URI::make_read_page_url($self->request->url,
                                                        $ancestors[-1])));
  }

  return @list;
}

sub _get_properties {
  my ($self, $key) = @_;

  $self->request->configuration->properties->get($key);
}

sub make_html_head_title {
  my $self = shift;

  my $name = $self->request->name;

  my $db = $self->db;
  my $subject = '';
  if ($db->exists_live_page($name)) {
    $subject = ' - ' . $db->page_subject($name);
  }

  my $prefix = $self->_get_properties('wikicker.name');
  return WiKicker::HTML::escape("$prefix: $name$subject");
}

sub stylesheet_uri_list {
  my $self = shift;

  $self->request->configuration->stylesheet_uri_list($self->request->url);
}

sub stylesheet_type {
  my $self = shift;

  if ($self->{stylesheet_type}) {
    return $self->{stylesheet_type};
  }
  my $stylesheet_type = $self->_get_properties('html.stylesheet.type');
  if (defined $stylesheet_type && $stylesheet_type ne '') {
    $self->{stylesheet_type} = $stylesheet_type;
  }
  else {
    $self->{stylesheet_type} = 'old';
  }

  return $self->{stylesheet_type};
}

sub external_head_scripts_list {
  my $self = shift;

  return $self->request->configuration->html_external_script_list;
}

sub make_html_header {
  my $self = shift;


  my $result = '';

  if ($self->stylesheet_type eq 'tDiary') {
    $result .= '<div class="main">';
  }

  my $name = $self->request->name;
  my $media = $self->request->param('media') || '';
  unless ($media eq 'print') {
    $result .= $self->make_header_toolbar($name);
  }
  $result .= $self->make_page_name_div;
  unless ($media eq 'print') {
    $result .= $self->make_page_toolbar($name);
  }

  $self->response->add_content($result);
}

sub _make_page_name_div_action_prefix {
  my $self = shift;

  my $action = $self->request->action;
  if ($self->request->attribute('action_label')) {
    $action = $self->request->attribute('action_label');
  }
  if ($action eq 'read') {
    $action = '';
  }
  else {
    $action = "$action: ";
  }
}

sub make_page_name_div {
  my $self = shift;

  my $result;
  $result .= qq(<div class="head-name">\n);
  if ($self->db->page_name_list->ancestors_page_name_list($self->request->name)) {
    $result .= $self->_make_page_name_div_navigation($self->request->name);
    $result .= $self->_make_page_name_div_title(WiKicker::PageName::main_name($self->request->name));
  }
  else {
    $result .= $self->_make_page_name_div_title($self->request->name);
  }
  if ($self->request->attribute('revision')) {
    $result .= q(This is old revision. ->);
    $result .= qq(<a href=") . $self->latest_url . qq(">latest</a>);
  }
  $result .= $self->make_header_last_modified;
  $result .= qq(\n</div>\n);

  return $result;
}

sub _head_hierarchy_class {
  my $self = shift;

  if ($self->stylesheet_type eq 'tDiary') {
    return 'adminmenu';
  }

  return 'head-hierarchy';
}

sub _make_page_name_div_navigation {
  my ($self, $name) = @_;

  my @path = split('/', $name);
  if (@path <= 1) {
    return '';
  }

  my @path_name;
  my @path_link;
  for (my $i = 0; $i < @path - 1; $i++) {
    push(@path_name, $path[$i]);
    push(@path_link,
         $self->link_maker->make_page_name_link(join('/', @path_name),
                                                '',
                                                $path[$i], 0));
  }

  my $search_url = $self->request->url . '?search=' .
    WiKicker::URI::escape_URI($name);

  my $class = $self->_head_hierarchy_class;

  my $result;
  $result .= qq(<div class="$class">[ );
  $result .= join(' &gt; ', @path_link);
  $result .= ' ] ';
  $result .= $self->_make_anchor($search_url, 'Search: ' . $name, $name);
  $result .= qq(</div>\n);

  return $result;
}

sub _make_anchor {
  my ($self, $url, $title, $content) = @_;

  $title   = WiKicker::HTML::escape_attribute($title);
  $content = WiKicker::HTML::escape($content);

  return qq(<a href="$url" title="$title">$content</a>);
}

sub _make_page_name_div_title {
  my ($self, $short_name) = @_;

  my $action = $self->_make_page_name_div_action_prefix;

  my $search_url
    = $self->request->url . '?search=' .
      WiKicker::URI::escape_URI($short_name);
  my $result;
  $result .= q(<h1>);
  $result .= WiKicker::HTML::escape($action);
  $result .= $self->_make_anchor($search_url, 'Search: ' . $short_name, $short_name);
  if ($self->request->attribute('revision')) {
    $result .= " (";
    $result .= WiKicker::HTML::escape($self->request->attribute('revision'));
    $result .= ")";
  }
  $result .= q(</h1>);
  return $result;
}

sub _header_last_modified_class {
  my $self = shift;

  if ($self->stylesheet_type eq 'tDiary') {
    return 'adminmenu';
  }

  return 'head-lastmodified';
}

sub make_header_last_modified {
  my $self = shift;

  my $db = $self->db;
  return '' unless ($self->enable_header_last_modified);

  my $result = '';
  my $last_modified = $db->last_modified($self->request->name);
  my $last_modified_string;
  if ($last_modified) {
    $last_modified_string
      = $self->request->language_handle->date_html_str($last_modified,
                                                       $self->timezone);
  }
  else {
    $last_modified_string = 'unknown';
  }

  my $revision = $self->request->attribute('latestrevision');
  my $author = $db->page_author($self->request->name);
  my $class = $self->_header_last_modified_class;
  $result .= qq(<div class="$class">\n);
  my $last_modified_text = $self->request->maketext('Last-Modified');
  $result .= qq($last_modified_text: $last_modified_string);
  if ($revision) {
    $result .= qq(; Revision: $revision);
  }
  if ($author) {
    $author = $self->link_maker->make_page_name_link($author, '', '', 0);
    $result .= qq(; by $author);
  }
  $result .= q(</div>);
  return $result;
}

sub _wrap_toolbar_item {
  my ($self, $text) = @_;

  if ($self->stylesheet_type eq 'tDiary') {
    return qq(<span class="adminmenu">$text</span>);
  }
  else {
    return $text;
  }
}

sub _make_header_toolbar_item {
  my ($self, $url, $text, $title) = @_;

  my $link_text = WiKicker::HTML::escape($self->request->maketext($text));
  if ($title ne '') {
    my $link_title = WiKicker::HTML::escape_attribute($self->request->maketext($title));
    return $self->_wrap_toolbar_item(qq(<a href="$url" title="$link_title">$link_text</a>));
  }
  else {
    return $self->_wrap_toolbar_item(qq(<a href="$url">$link_text</a>));
  }
}

sub make_header_toolbar {
  my ($self, $name) = @_;

  my $div_class;
  if ($self->stylesheet_type eq 'tDiary') {
    $div_class = 'adminmenu';
  }
  else {
    $div_class = 'head-toolbar';
  }

  my $result = qq(\n<!-- begin header toolbar -->\n<div class="$div_class">\n);
  my $query = $self->request->query;
  $result .= $query->start_form(-action => $self->request->url, -method=> 'POST');
  $result .= qq(  <div>\n   );

  my $url = $self->request->url;

  my @toolbar_item;
  push(@toolbar_item,
       $self->_make_header_toolbar_item($url, 'FrontPage', 'Go top page'));
  push(@toolbar_item,
       $self->_make_header_toolbar_item("$url/RecentChanges.html",
                                        'changes',
                                        'recent changes'));
  push(@toolbar_item,
       $self->_make_header_toolbar_item($self->index_page_url,
                                        'index',
                                        'page index'));
  if ($self->enable_header_toolbar_create) {
    push(@toolbar_item,
         $self->_make_header_toolbar_item("$url/CreatePage.html",
                                          'create',
                                          'create new page'));
  }

  push(@toolbar_item,
       $self->_make_header_toolbar_item("$url/SearchPage.html",
                                        'search_button',
                                        'search the WikiForum'));
  push(@toolbar_item,
       $self->_make_header_toolbar_item("$url/PreferencePage.html",
                                        'preferences_button',
                                        'preferences'));

  my $user_name = $self->request->user->name;
  if ($user_name && $self->db->exists_live_page($user_name)) {
    push(@toolbar_item,
         $self->_wrap_toolbar_item($self->link_maker
                                   ->make_page_name_link($user_name,
                                                         '',
                                                         '', 0)));
  }
  $result .= join(qq(\n | ), @toolbar_item);
  $result .= qq(\n | );

  $result .= $query->textfield(-name => 'name',
                               -title => 'Input PageName or search query',
                               -size => 14);
  $result .= $query->submit(-name => 'action', -value => 'read',
                            -title => 'Go to the page');
  $result .= $query->submit(-name => 'action', -value => 'search',
                            -title => 'Search the page');
  $result .= qq(\n  </div>);
  $result .= $query->end_form;

  $result .= qq(\n</div><!-- end $div_class -->\n<!-- end header toolbar -->\n);

  return $result;
}

sub make_page_toolbar {
  my ($self, $name) = @_;

  my @link;
  push(@link, $self->make_page_toolbar_edit($name));
  push(@link, $self->make_page_toolbar_copy($name));
  push(@link, $self->make_page_toolbar_diff($name));
  push(@link, $self->make_page_toolbar_latest($name));
  push(@link, $self->make_page_toolbar_rlog($name));
  push(@link, $self->make_page_toolbar_raw($name));

  my $link_string;

  if ($self->stylesheet_type eq 'tDiary') {
    $link_string = join("\n| ", map { $self->_wrap_toolbar_item($_) } grep { $_ ne '' } @link);
  }
  else {
    $link_string = join("\n| ", grep { $_ ne '' } @link);
  }

  return '' unless $link_string;

  if ($self->stylesheet_type eq 'tDiary') {
    return qq(<div class="adminmenu">\n$link_string</div><!-- end adminmenu -->\n);
  }
  else {
    return qq(<div class="page-toolbar">\n$link_string</div>\n);
  }
}

sub make_page_toolbar_edit {
  my ($self, $name) = @_;
  if ($self->enable_header_toolbar_edit) {
    return $self->
      _make_localized_link($self->link_maker->edit_uri($self->request->name),
                           'edit',
                           'edit this page');
  }
  return '';
}

sub make_page_toolbar_copy {
  my ($self, $name) = @_;

  if ($self->enable_header_toolbar_copy) {
    return $self->_make_localized_link($self->copy_url,
                                       'copy',
                                       'copy this page');
  }
  return '';
}

sub make_page_toolbar_diff {
  my ($self, $name) = @_;

  if ($self->enable_header_toolbar_diff) {
    return $self->_make_localized_link($self->diff_url,
                                       'diff',
                                       'show difference');
  }
  else {
    return '';
  }
}

sub make_page_toolbar_latest {
  my ($self, $name) = @_;

  if ($self->request->attribute('revision')) {
    my $latest_url = $self->latest_url;
    return qq(<a href="$latest_url">latest</a>);
  }
  else {
    return '';
  }
}

sub make_page_toolbar_rlog {
  my ($self, $name) = @_;

  if ($self->enable_header_toolbar_rlog) {
    my $rlog_url = $self->rlog_url;
    return qq(<a href="$rlog_url">rlog</a>);
  }
  else {
    return '';
  }
}

sub make_page_toolbar_raw {
  my ($self, $name) = @_;

  if ($self->enable_header_toolbar_raw) {
    my $raw_url = $self->raw_url;
    return qq(<a href="$raw_url" title="source of this WikiPage">raw</a>);
  }
  else {
    return '';
  }
}

# sub track_back_ping_url {
#   my $self = shift;

#   if ($self->request->attribute('normalpage')
#       && $self->request->action eq 'read') {
#     return WiKicker::URI::make_track_back_url($self->request->query->url,
#                                               $self->request->name);
#   }
#   else {
#     '';
#   }
# }

# sub make_track_back_div {
#   my $self = shift;

#   my $entry_map = $self->db->cache('trackback', $self->request->name);
#   unless (defined $entry_map) {
#     return '';
#   }
#   $entry_map = Storable::thaw($entry_map);
#   my @entries = $entry_map->get_entries_list;

#   my $result = '<h2>TrackBack</h2>';
#   $result .= '<div class="trackback">';
#   if (@entries) {
#     $result .= '<dl>';
#     for my $entry (@entries) {
#       my $escaped_url = WiKicker::HTML::escape($entry->url);
#       my $href_url = WiKicker::HTML::escape_attribute($entry->url);
#       my $title = $entry->title;
#       if ($title) {
#         $title = WiKicker::HTML::escape($title);
#       }
#       else {
#         $title = $escaped_url;
#       }

#       $result .= '<dt>';
#       unless ($href_url =~ /^(java-script|vbscript|about):/i) {
#         $result .= qq(<a href="$href_url">$title</a>);
#       }
#       else {
#         $result .= qq($title);
#       }
#       $result .= '</dt>';
#       if ($entry->excerpt) {
#         $result .= q(<dd>excerpt: );
#         $result .= WiKicker::HTML::escape($entry->excerpt);
#         $result .= q(</dd>);
#       }
#       if ($entry->blog_name) {
#         $result .= q(<dd>blog_name: );
#         $result .= WiKicker::HTML::escape($entry->blog_name);
#         $result .= q(</dd>);
#       }
#       if ($entry->title) {
#         $result .= qq(<dd>url: <a href="$href_url">$escaped_url</a></dd>);
#       }
#       $result .= '<dd>time: ';
#       $result .=  WiKicker::Time::japanese_date_html_str($entry->timestamp,
#                                                          $self->timezone);
#       $result .= '</dd>';

#     }
#     $result .= '</dl></div>';
#   }
#   return $result;
# }

sub make_html_footer {
  my $self = shift;

  my $result = $self->_make_html_footer_from_text;

  if ($self->stylesheet_type eq 'tDiary') {
    $result .= '</div>'; #div.main
  }

  $result .= '</body></html>';

  $self->response->add_content($result);
}

sub _make_html_footer_from_text {
  my $self = shift;

  my $short_name = $self->request->name;
  $short_name =~ s|^(.*)/||;
  my $short_name_attribute = WiKicker::HTML::escape_attribute($short_name);
  my $short_name_urlencoded = WiKicker::URI::encode_x_www_form_urlencoded($short_name);


  my $footer = $self->load_footer_file('html.footer.file',
                                       '/WiKicker/Template/HTML/footer.txt');

  $footer =~ s/\[\[value:ShortNameAttribute\]\]/$short_name_attribute/g;
  $footer =~ s/\[\[value:ShortNameUrlEncoded\]\]/$short_name_urlencoded/g;
  $footer =~ s/\[\[value:ShortName\]\]/$short_name/g;

  $footer = $self->common_footer_text_filter($footer);
  return $footer;
}

# sub _make_html_footer_by_template {
#   my $self = shift;

#   my $load_average_result = `uptime`;
#   $load_average_result =~ /(load.*)$/;
#   my $load_average = $1 || '';
#   my $process_time = "Process Time: " . tv_interval($self->{begin_time}) . "s";
#   my $short_name = $self->request->name;
#   $short_name =~ s|^(.*)/||;
#   my $short_name_attribute = WiKicker::HTML::escape_attribute($short_name);

#   my @include = map {$_ . '/WiKicker/Template' } @INC;

#   my $config = { INCLUDE_PATH => [ @include ] };
#   WiKicker::Loader::load_module("Template");
#   my $template = Template->new($config);
#   my $vars = { load_average => $load_average,
#                process_time => $process_time,
#                short_name_attribute => $short_name_attribute,
#                version => $WiKicker::VERSION,
#              };
#   my $template_out = '';
#   $template->process('HTML/footer.tt2', $vars, \$template_out);
#   return $template_out;
# }

sub enable_header_toolbar_edit {
  return 0;
}

sub enable_header_toolbar_copy {
  return 0;
}

sub enable_header_toolbar_create {
  return 0;
}

sub enable_header_toolbar_rlog {
  return 0;
}

sub enable_header_toolbar_raw {
  return 0;
}

sub enable_header_toolbar_diff {
  return 0;
}

sub enable_header_last_modified {
  my $self = shift;

  if ($self->request->attribute('editable')) {
    return 1;
  }
  else {
    return 0;
  }
}

sub copy_url {
  my $self = shift;

  $self->request->url
    . '?name=CopyPage&amp;from='
      . WiKicker::URI::escape_URI($self->request->name);
}

sub rlog_url {
  my $self = shift;

  $self->request->url .
    '?rlog=' . WiKicker::URI::escape_URI($self->request->name);
}

sub diff_url {
  my $self = shift;

  $self->request->url .
    '?diff=' . WiKicker::URI::escape_URI($self->request->name);
}

sub raw_url {
  my $self = shift;

  my $revision_param = '';
  if ($self->request->attribute('revision')) {
    $revision_param = '&revision=';
    $revision_param .= WiKicker::URI::escape_URI($self->request->attribute('revision'));
  }
  return $self->request->url . '?name=' .
    WiKicker::URI::escape_URI($self->request->name) . '&amp;format=raw'
        . $revision_param;
}

sub latest_url {
  my $self = shift;

  $self->request->url .'?name=' .
    WiKicker::URI::escape_URI($self->request->name);
}

sub timezone {
  my $self = shift;

  return $self->request->user->timezone
    || $self->_get_properties('user.timezone');
}

sub front_page_url {
  my $self = shift;

  return $self->request->url;
}

sub index_page_url {
  my $self = shift;

  return $self->request->url . '/IndexPage.html';
}

sub _make_localized_link {
  my ($self, $url, $text, $title) = @_;

  my $L10N_title
    = WiKicker::HTML::escape_attribute($self->request->maketext($title));
  my $L10N_text
    = WiKicker::HTML::escape($self->request->maketext($text));

  return qq(<a href="$url" title="$L10N_title">$L10N_text</a>);
}

sub _make_document_node;
sub _add_and_flush_html_fragment;
sub _create_html_env;
sub _create_html_fragment_visitor;

1;

__END__

sub _make_document_node {
  my ($self, $wiki_page) = @_;

  return
    $self->request->environment->create_document_parser->parse($wiki_page);
}

sub _add_and_flush_html_fragment {
  my ($self, $document_node, $no_edit_section) = @_;

  my $visitor = $self->_create_html_fragment_visitor($no_edit_section);
  $visitor->visit($document_node);
  $visitor->content;
  #-IFDEBUG: WiKicker::cycle_check($visitor);
}

sub _create_html_fragment_visitor {
  my ($self, $no_edit_section) = @_;

  my $no_automatic_link
    = ! $self->request->configuration->automatic_link_enabled($self->request->load_average);

  my $visitor;
  if ($self->stylesheet_type eq 'tDiary') {
    $visitor = WiKicker::WikiPage::tDiaryFragmentVisitor->new;
  }
  else {
    $visitor = WiKicker::WikiPage::WikiPageHtmlFragmentVisitor->new;
  }

  $visitor->env($self->_create_html_env($no_automatic_link));
  $visitor->no_edit_section($no_edit_section);
  $visitor->no_automatic_link($no_automatic_link);
  my $response = $self->response;
  $visitor->flush_closure(sub { $response->add_content(shift);
                                if ($response->content_length > 512) {
                                  $response->flush;
                                }
                              });
  return $visitor;
}

sub _create_html_env {
  my ($self, $no_automatic_link) = @_;

  my $extraset = [];
  unless ($no_automatic_link) {
    $extraset = $self->request->configuration->automaticlink_extraset_list;
  }

  my $env = WiKicker::WikiPage::HtmlEnv
    ->new(user => $self->request->user,
          resolver => $self->request->environment->resolver,
          wikicker_url => $self->request->url,
          wri_factory => $self->request->environment->create_wri_factory,
          configuration => $self->request->configuration,
          environment => $self->request->environment,
          query => $self->request->query,
          page_name => $self->request->name,
          localizer => $self->request,
          extraset => $extraset,
          page_name_list => $self->db->page_name_list,
         );
  $env->link_maker($self->link_maker);

  return $env;
}
