#
# WiKicker
#
#  Index.pm
#
#
# Copyright (C) 2003,2004 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Index.pm,v 1.4 2004/05/01 07:06:08 naney Exp $
#
#

package WiKicker::WikiCGI::View::HTML::Index;

use 5.005;
use strict;

use WiKicker::WikiCGI::View::HTML;
use WiKicker::PageName;

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiCGI::View::HTML);

sub make_html_body {
  my $self = shift;

  my $db = $self->db;
  my $content;

  my $offset = $self->request->attribute('offset');
  my $last = $offset + @{$self->request->attribute('list')} - 1;
  my $list_size = $self->request->attribute('list_size');

  my $navigation = $self->_make_navigation($list_size,
                                           $offset,
                                           $self->request->attribute('item_max'));
  $offset++;
  $last++;

  $content .= qq(<div>$offset - $last of $list_size. $navigation</div>\n);

  $content .= qq(<ul class="index">\n);
  for my $name (@{$self->request->attribute('list')}) {
    my $link = $self->link_maker->make_page_name_link($name, '', '', 0);
    my $subject = $db->page_subject($name) || '';
    if ($subject) {
      $subject = ' - ' . WiKicker::HTML::escape($subject);
    }
    my $author = $db->page_author($name) || '';
    if ($author) {
      $author = ' (' . WiKicker::HTML::escape($author) . ')';
    }
    $content .= qq(<li>$link$author$subject</li>\n);
  }
  $content .= qq(</ul>\n);
  $content .= qq(<div>$navigation</div>\n);

  $self->response->add_content($content);
}

sub _make_navigation {
  my ($self, $list_size, $offset, $item_max) = @_;

  my @link;
  my $index = 0;
  for (my $start = 0; $start < $list_size; $start += $item_max) {
    $index++;
    if ($start <= $offset && $offset < $start + $item_max) {
      push(@link, qq([$index]));
    }
    else {
      push(@link, qq([<a href="?offset=$start">$index</a>]));
    }
  }
  return join(' ', @link);
}

1;
