#
# WiKicker
#
#  CommitProcessorVisitor.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: CommitProcessorVisitor.pm,v 1.9 2004/08/03 12:55:52 naney Exp $
#
#

package WiKicker::WikiPage::CommitProcessorVisitor;

use strict;

use Carp;
use WiKicker::PageName;
use WiKicker::WikiPage::TraverseVisitor;
use WiKicker::WRI::Resolver;
use WiKicker::WikiPage::Env;
use WiKicker::Time;

use vars qw(@ISA);
@ISA = qw(WiKicker::WikiPage::TraverseVisitor);

sub new {
  my ($class, %param) = @_;

  croak 'no wri_factory' unless defined $param{wri_factory};

  my $self = bless { now => time(), wri_factory => $param{wri_factory} }, $class;
}

sub resolver {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{resolver} = $value;
  }
  return $self->{resolver};
}

sub env {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{env} = $value;
  }
  return $self->{env};
}

sub visit_BracketNameNode {
  my ($self, $bracket_name_node) = @_;

  my $wri = $bracket_name_node->wri;
  if ($wri->isa('WiKicker::WRI::RWRN')) {
    if ($wri->scheme eq 'gmtime' && $wri->rest eq 'now') {
      $self->_set_gmtime_now($bracket_name_node);
    }
    elsif ($wri->scheme eq 'datetime' && $wri->rest eq 'now') {
      $self->_set_datetime_now($bracket_name_node);
    }
  }
  unless ($bracket_name_node->original =~ /\[\]\]$/) {
    return;
  }
  unless ($self->resolver) {
    $self->resolver(WiKicker::WRI::Resolver->new);
  }
  unless ($self->env) {
    $self->env(WiKicker::WikiPage::Env->new(resolver => $self->resolver,
                                            wri_factory => $self->{wri_factory},
                                           ));
  }
  my $env = $self->env;
  my $url = $bracket_name_node->wri->get_link_url($env);
  eval 'use WiKicker::Tool::GetUriTitle';
  if ($@) {
    croak "Can't load WiKicker::Tool::GetUriTitle: $@";
  }
  my $tool = WiKicker::Tool::GetUriTitle->new;
  my $title = $tool->get_bracketed_title($url);
  $title =~ s/^\[Amazon\.co\.jp.*?:\s*/[/;
  $bracket_name_node->original('[[' . $bracket_name_node->reference . ']' . $title . ']');
}

sub _set_gmtime_now {
  my ($self, $bracket_name_node) = @_;

  my $new_wri = $self->{wri_factory}->create_wri('gmtime:' . $self->{now});
  $bracket_name_node->wri($new_wri);
  if ($bracket_name_node->content) {
    $bracket_name_node->original('[[gmtime:' . $self->{now} . '][' . $bracket_name_node->content . ']]');
  }
  else {
    $bracket_name_node->original('[[gmtime:' . $self->{now} . ']]');
  }
}

sub _set_datetime_now {
  my ($self, $bracket_name_node) = @_;

  my $dtf = WiKicker::Time::gmtime_to_W3CDTF_UTC($self->{now});
  my $new_wri = $self->{wri_factory}->create_wri('datetime:' . $dtf);
  $bracket_name_node->wri($new_wri);
  if ($bracket_name_node->content) {
    $bracket_name_node->original('[[datetime:' . $dtf . '][' . $bracket_name_node->content . ']]');
  }
  else {
    $bracket_name_node->original('[[datetime:' . $dtf . ']]');
  }
}

1;
