#
# WiKicker
#
#  Env.pm
#
#
# Copyright (C) 2003,2004,2005 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: Env.pm,v 1.8 2005/04/04 14:06:59 naney Exp $
#
#

package WiKicker::WikiPage::Env;

sub new {
  my ($class, %param) = @_;

  my $self = bless {}, $class;
  for my $key (keys %param) {
    $self->{$key} = $param{$key};
  }

  return $self;
}

sub wikicker_url   { $_[0]->{wikicker_url} }
sub page_name      { $_[0]->{page_name} }
sub page_name_list { $_[0]->{page_name_list} }
sub resolver       { $_[0]->{resolver} }
sub wri_factory    { $_[0]->{wri_factory} }
sub environment    { $_[0]->{environment} }
sub configuration  { $_[0]->environment->configuration }
sub db             { $_[0]->environment->db }

sub user {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{user} = $value;
  }
  return $self->{user};
}

package WiKicker::WikiPage::HtmlEnv;

use Carp;

use vars qw(@ISA %required_param);
BEGIN {
  @ISA = qw(WiKicker::WikiPage::Env);
  %required_param = ( localizer => '');
}

sub new {
  my ($class, %param) = @_;

  for my $p (keys %required_param) {
    croak "$class: No $p param" unless exists $param{$p};
  }

  my $self = bless WiKicker::WikiPage::Env->new(%param), $class;
  $self->{comment_number} = -1;

  return $self;
}

# localizer must have 'maketext' and 'language_handle' method.
sub localizer { shift->{localizer} }

sub in_inline {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{in_inline} = $value;
  }
  return $self->{in_inline};
}

sub comment_number {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{comment_number} = $value;
  }
  return $self->{comment_number};
}

sub query {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{query} = $value;
  }
  return $self->{query};
}

sub form_charset {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{form_charset} = $value;
  }
  return $self->{form_charset};
}

sub link_maker {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{link_maker} = $value;
  }
  return $self->{link_maker};
}

sub extraset {
  my ($self, $extraset) = @_;

  if (defined $value) {
    $self->{extraset} = $value;
  }
  return $self->{extraset};
}

sub add_explicit_term {
  my ($self, @linked) = @_;

  for my $name (@linked) {
    $self->{explicit_term_set}->{$name}++;
  }
}

sub add_implicit_term {
  my ($self, @linked) = @_;

  for my $name (@linked) {
    $self->{implicit_term_set}->{$name}++;
  }
}

sub explicit_term_list {
  my $self = shift;

  return [] unless exists $self->{explicit_term_set};
  return [ keys %{$self->{explicit_term_set}} ];
}

sub implicit_term_list {
  my $self = shift;

  return [] unless exists $self->{implicit_term_set};
  return [ keys %{$self->{implicit_term_set}} ];
}


1;
