#
# WiKicker
#
#  GetPageLinkVisitor.pm
#
#
# Copyright (C) 2003 WATANABE Yoshimasa <naney@naney.org>
#
# This file is part of the WiKicker.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc., 59
# Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#
# $Id: GetPageLinkVisitor.pm,v 1.3 2003/06/12 04:10:09 naney Exp $
#
#
package WiKicker::WikiPage::GetPageLinkVisitor;

use strict;

use WiKicker::PageName;
use WiKicker::WikiPage::TraverseVisitor;
use vars qw(@ISA);
@ISA = qw(WiKicker::WikiPage::TraverseVisitor);

sub new {
  my $class = shift;

  my $self = bless { page_link => [], page_name => [] }, $class;
}

sub visit_WriNode {
  my ($self, $wri_node) = @_;

  $self->_check_wri($wri_node->wri);
}

sub visit_BracketNameNode {
  my ($self, $bracket_name_node) = @_;
  $self->_check_wri($bracket_name_node->wri);
}

sub _check_wri {
  my ($self, $wri) = @_;

  if ($wri->isa('WiKicker::WRI::PWRN')) {
    push(@{$self->{page_link}}, $wri);
    push(@{$self->{page_name}}, $wri->page_name);
    return;
  }
  if ($wri->isa('WiKicker::WRI::RWRN')) {
    if ($wri->scheme eq 'page' ||
        $wri->scheme eq 'mtimelink' ||
        $wri->scheme eq 'page-subject' ||
        $wri->scheme eq 'page-mtime') {
      push(@{$self->{page_link}}, $wri);
      push(@{$self->{page_name}}, $wri->rest);
    }
  }
}

sub page_link {
  my $self = shift;

  return @{$self->{page_link}};
}

sub page_name_list {
  my $self = shift;

  return @{$self->{page_name}};
}

1;
